@@ -120,12 +120,12 @@ class HTTPPasswordHandler(object):
 
         # Loading .hg/hgrc [auth] section contents. If prefix is given,
         # it will be used as a key to lookup password in the keyring.
-        auth_user, pwd, prefix_url = self.load_hgrc_auth(ui, base_url)
+        auth_user, pwd, prefix_url = self.load_hgrc_auth(ui, base_url, user)
         if prefix_url:
             keyring_url = prefix_url
         else:
             keyring_url = base_url
-        ui.debug("keyring URL: %s\n" % keyring_url)
+        ui.debug("Keyring URL: %s\n" % keyring_url)
 
         # Checking the memory cache (there may be many http calls per command)
         cache_key = (realm, keyring_url)
@@ -162,6 +162,8 @@ class HTTPPasswordHandler(object):
                                   base_url, user, pwd)
                 self.last_reply = dict(realm=realm,authuri=authuri,user=user)
                 return user, pwd
+            else:
+                ui.debug(_("Password not present in the keyring\n"))
 
         # Is the username permanently set?
         fixed_user = (user and True or False)
@@ -197,7 +199,7 @@ class HTTPPasswordHandler(object):
         self.last_reply = dict(realm=realm,authuri=authuri,user=user)
         return user, pwd
 
-    def load_hgrc_auth(self, ui, base_url):
+    def load_hgrc_auth(self, ui, base_url, user):
         """
         Loading [auth] section contents from local .hgrc
 
@@ -235,11 +237,17 @@ class HTTPPasswordHandler(object):
         except AttributeError:
             try:
                 # hg 1.8
-                from mercurial.url import readauthforuri
-            except ImportError:
+                import mercurial.url
+                readauthforuri = mercurial.url.readauthforuri
+            except (ImportError, AttributeError):
                 # hg 1.9
-                from mercurial.httpconnection import readauthforuri
-            res = readauthforuri(local_ui, base_url)
+                import mercurial.httpconnection
+                readauthforuri = mercurial.httpconnection.readauthforuri
+            if readauthforuri.func_code.co_argcount == 3:
+                # Since hg.0593e8f81c71
+                res = readauthforuri(local_ui, base_url, user)
+            else:
+                res = readauthforuri(local_ui, base_url)
             if res:
                 group, auth_token = res
             else:
